(function () {

    rwt.chart = {};

    rwt.chart.Apexchart = function (parent, id, options, renderer) {
        this._data = [];
        this._element = this.createElement(parent, id);
        this._id = id;
        this._options = JSON.parse(options);
        if (renderer !== undefined) {
            this._renderer = renderer(this);
            this._renderer.updateOptions(this._options);
        }
        iterate(this._options);
        this._chartApex = {};

        rap.on("render", function () {
            if (this._needsRender) {
             //   console.log('render : ' + this._id + " " + this._options.chart.type);
                var chartApex = new ApexCharts(document.querySelector("#chartApex" + this._id), this._options);
                this._chartApex = chartApex;
                chartApex.render();
                this._needsRender = false;
               // console.log('stop render : ' + this._id);
            }
        }.bind(this));

        parent.addListener("Resize", function () {
            this._resize(parent.getClientArea());
        }.bind(this));

        this._resize(parent.getClientArea());
    };

    rwt.chart.Apexchart.prototype = {

        createElement: function (parent, id) {
            var element = document.createElement("div");
            element.style.position = "absolute";
            element.style.left = "0";
            element.style.top = "0";
            element.style.width = "100%";
            element.style.height = "100%";
            element.id = "chartApex" + id;
            parent.append(element);
            return element;
        },

        // notifySelection: function (index, detail, text) {
        //     var remoteObject = rap.getRemoteObject(this);
        //     var params = {"index": index};
        //     if (arguments.length > 1) {
        //         params.detail = detail;
        //     }
        //     if (arguments.length > 2) {
        //         params.text = text;
        //     }
        //     remoteObject.notify("Selection", params);
        // },

        _resize: function (clientArea) {
            this._needsRender = true;
        },

        destroy: function () {
            var element = this._element;
            if (element.parentNode) {
                this._chartApex.destroy();
                element.parentNode.removeChild(element);
            }
        }

    };

    // UTILITIES
    rwt.chart._renderers = {};
    //
    rwt.chart.register = function (name, generator) {
        if (name in rwt.chart._renderers) {
            throw new Error("Chart type already registered: " + name);
        }
        rwt.chart._renderers[name] = generator;
    };


    rwt.chart.loadCss = function (url) {
        var element = document.createElement("link");
        element.rel = "stylesheet";
        element.type = "text/css";
        element.media = "screen";
        element.href = url;
        document.getElementsByTagName("head")[0].appendChild(element);
    };


    // TYPE HANDLER
    rap.registerTypeHandler("rwt.chart.Apexchart", {

        factory: function (properties) {
            return new rwt.chart.Apexchart(rap.getObject(properties.parent), properties.id_chart, properties.options, rwt.chart._renderers[properties.renderer]);
        },

        destructor: "destroy",
    });

})();

function iterate(options) {
    Object.keys(options).forEach(key => {
        if (typeof options[key] === 'object') {
            iterate(options[key])
        }
        if (options[key] === "undefined")
            options[key] = undefined;
    })
}

function getSizeStorage(size, round) {
    if (size == null)
        return '';

    if (size < 1024)
        return size.toFixed(0) + ' B';

    if (size < Math.pow(1024, 2))
        return (size / 1024).toFixed(round) + ' KB';

    if (size < Math.pow(1024, 3))
        return (size / 1024 / 1024).toFixed(round) + ' MB';

    if (size < Math.pow(1024, 4))
        return (size / 1024 / 1024 / 1024).toFixed(round) + ' GB';

    if (size < Math.pow(1024, 5))
        return (size / 1024 / 1024 / 1024 / 1024).toFixed(round) + ' TB';

    return (size / 1024 / 1024 / 1024 / 1024 / 1024).toFixed(round) + ' PT';
};
